/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "CAENCfdController.h"
#include "drivers/global/DriversCommands.h"
#include "controllers/common/family/Families.h"
#include <boost/thread/thread.hpp>
#include <controllers/common/acquisition/mode/DPPAcquisitionCommon.h>
#include "controllers/common/acquisition/mode/ListModeTypes.h"

using namespace std;
using namespace boost;

namespace npp {

const string CAENCfdController::TYPE = "cfd_controller";

/*
 * Constructor
 */
CAENCfdController::CAENCfdController(const string& name) : CAENPsdController(name) {

	setFamily(family::ACQUISITION, family::SETTING);

	// Register properties
	// CFD parameters
	delay.init(this, NOSAVE, "delay");
	attenuation.init(this, NOSAVE, "attenuation");

	// Energy filter parameters
	trapezoidRise.init(this, NOSAVE, "trapezoid_rise");
	trapezoidFlat.init(this, NOSAVE, "trapezoid_flat");
	decayTime.init(this, NOSAVE, "decay_time");
	peaking.init(this, NOSAVE, "peaking");
	bitShift.init(this, NOSAVE, "bit_shift");
	channelOffset.init(this, SAVE, "channel_offset");

	m_CfdDriver.init(this, "cfd_driver");

	registerFunction(TYPE);
}

/*
 * Destructor
 */
CAENCfdController::~CAENCfdController() {

}

/*
 * postConfiguration
 */
void CAENCfdController::postConfiguration() {
	CAENPsdController::postConfiguration();
	registerRefresher(delay, &CAENCfdController::refreshDelayProperty, this);
	registerRefresher(attenuation, &CAENCfdController::refreshAttenuationProperty, this);
	registerRefresher(pulsePolarity, &CAENCfdController::refreshPulsePolarityProperty, this);
	registerRefresher(trapezoidRise, &CAENCfdController::refreshTrapezoidRiseProperty, this);
	registerRefresher(trapezoidFlat, &CAENCfdController::refreshTrapezoidFlatProperty, this);
	registerRefresher(decayTime, &CAENCfdController::refreshDecayTimeProperty, this);
	registerRefresher(peaking, &CAENCfdController::refreshPeakingProperty, this);
	registerRefresher(bitShift, &CAENCfdController::refreshBitShiftProperty, this);
	registerRefresher(channelOffset, &CAENCfdController::refreshChannelOffsetProperty, this);
	updateCfdProperties(displayChannel() + channelOffset());
	cardType = CFD_BOARD_TYPE;
}

void CAENCfdController::refreshDelayProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->delay = aValue;
}

void CAENCfdController::refreshAttenuationProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->attenuation = aValue;
}

void CAENCfdController::refreshPulsePolarityProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->pulsePolarity.set(displayChannel() + channelOffset(), aValue);
	CAENPsdController::refreshPulsePolarityProperty(aValue);
}

void CAENCfdController::refreshTrapezoidRiseProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->trapezoidRise.set(displayChannel() + channelOffset(), aValue);
}

void CAENCfdController::refreshTrapezoidFlatProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->trapezoidFlat.set(displayChannel() + channelOffset(), aValue);
}

void CAENCfdController::refreshDecayTimeProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->decayTime.set(displayChannel() + channelOffset(), aValue);
}

void CAENCfdController::refreshPeakingProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->peaking.set(displayChannel() + channelOffset(), aValue);
}

void CAENCfdController::refreshBitShiftProperty(int32 aValue) throw (CannotSetValue) {
	m_CfdDriver->bitShift.set(displayChannel() + channelOffset(), aValue);
}

void CAENCfdController::refreshDisplayChannelProperty(int32 aChannel) throw (CannotSetValue) {
	updateCfdProperties(aChannel + channelOffset());
	CAENPsdController::refreshDisplayChannelProperty(aChannel);
}

void CAENCfdController::refreshChannelOffsetProperty(int32 aChannel) throw (CannotSetValue) {
	updateCfdProperties(displayChannel() + aChannel);
	CAENPsdController::refreshDisplayChannelProperty(displayChannel());
}

void CAENCfdController::refreshNbChannelsProperty(int32 aValue)  throw (CannotSetValue) {
	CAENPsdController::refreshNbChannelsProperty(aValue);
	m_CfdDriver->oldNbChannels = nbChannels();
	m_CfdDriver->newNbChannels = aValue;
}

void CAENCfdController::refreshDppAcqModeProperty(int32 aValue)  throw (CannotSetValue) {
	CAENAdcController::refreshDppAcqModeProperty(aValue);
	if (aValue == acquisition::DPPAcquisitionCommon::SCOPE_MODE) {
		m_CfdDriver->useScope = 1;
		m_CfdDriver->useListMode = 0;
	}
	else {
		m_CfdDriver->useScope = 0;
		m_CfdDriver->useListMode = 1;
	}
	m_dgtzDriver->dppAcqMode = acquisition::DPPAcquisitionCommon::SCOPE_MODE;
	boost::thread td(boost::bind(&CAENPsdController::initBoard, this));
}

void CAENCfdController::updateBoardProperties() {
	modelName.update(m_dgtzDriver->modelName());
	channels.update(m_dgtzDriver->channels());
	nbChannels.update(m_dgtzDriver->nbChannels());
	minRange = 0;
	maxRange = m_dgtzDriver->nbChannels() - 1;
	if (m_CfdDriver->useScope() == 1) {
		dppAcqMode.update(acquisition::DPPAcquisitionCommon::SCOPE_MODE);
	}
	else {
		dppAcqMode.update(acquisition::DPPAcquisitionCommon::LISTMODE_MODE);
	}
}

void CAENCfdController::updateCfdProperties(int32 channel) {
	delay.update(m_CfdDriver->delay());
	attenuation.update(m_CfdDriver->attenuation());
	pulsePolarity.update(m_CfdDriver->pulsePolarity.get(channel));
	trapezoidRise.update(m_CfdDriver->trapezoidRise.get(channel));
	trapezoidFlat.update(m_CfdDriver->trapezoidFlat.get(channel));
	decayTime.update(m_CfdDriver->decayTime.get(channel));
	peaking.update(m_CfdDriver->peaking.get(channel));
	bitShift.update(m_CfdDriver->bitShift.get(channel));
}

}
