/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>

#include "drivers/stereo_elec/StereoElecDef.h"
#include "drivers/stereo_elec/STDAQParams.h"
#include "FEParamController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_daq {

const string FEParamController::TYPE = "feparam_controller";


/*
 * Constructor
 */
FEParamController::FEParamController(const string& name) :
	ExperimentController(name), Select(this),Set(this) {

	setFamily(family::ACQUISITION);

	// Init property
	nfeboard.init(this, NOSAVE, "nfeboard");
	ntot.init(this, NOSAVE, "ntot");
	ntail.init(this, NOSAVE, "ntail");
	pretrig.init(this, NOSAVE, "pretrig");
	cfddelay.init(this, NOSAVE, "cfddelay");
	cfdfraction.init(this, NOSAVE, "cfdfraction");
	trigmask.init(this, NOSAVE, "trigmask");
	trigmode.init(this, NOSAVE, "trigmode");
	slotindex.init(this, NOSAVE, "slotindex");

	itotchannel.init(this, NOSAVE, "itotchannel");
	islot.init(this, NOSAVE, "islot");
	ichannel.init(this, NOSAVE, "ichannel");
	namechannel.init(this, NOSAVE, "namechannel");
	gain10.init(this, NOSAVE, "gain10");
    trigthres.init(this, NOSAVE, "trigthres");
	cfdthres.init(this, NOSAVE, "cfdthres");
	isselected.init(this, NOSAVE, "isselected");

	selectchoice.init(this,NOSAVE,"selectchoice");
	selectslot.init(this,NOSAVE,"selectslot");
	setparam.init(this, NOSAVE, "setparam");
	setvalue.init(this, NOSAVE, "setvalue");

	m_Driver.init(this, "driver");

	m_StartActivated = false;

	registerFunction(TYPE);


}

FEParamController::FEParamController(const FEParamController& controller) :
	ExperimentController(controller), Select(this),Set(this) {

	nfeboard.copy(this, controller.nfeboard);
	ntot.copy(this, controller.ntot);
	ntail.copy(this, controller.ntail);
	pretrig.copy(this, controller.pretrig);
	cfddelay.copy(this, controller.cfddelay);
	cfdfraction.copy(this, controller.cfdfraction);
	trigmask.copy(this, controller.trigmask);
	trigmode.copy(this, controller.trigmode);
	slotindex.copy(this, controller.slotindex);

	itotchannel.copy(this, controller.itotchannel);
	islot.copy(this, controller.islot);
	ichannel.copy(this, controller.ichannel);
	namechannel.copy(this, controller.namechannel);
	gain10.copy(this, controller.gain10);
	trigthres.copy(this, controller.trigthres);
	cfdthres.copy(this, controller.cfdthres);
	isselected.copy(this, controller.isselected);

	selectchoice.copy(this, controller.selectchoice);
	selectslot.copy(this, controller.selectslot);
	setparam.copy(this, controller.setparam);
	setvalue.copy(this, controller.setvalue);

	m_Driver.copy(controller.m_Driver);

	m_StartActivated = false;
}

/*
 * Destructor
 */
FEParamController::~FEParamController() {
	// Empty
}

void FEParamController::postConfiguration() {

	registerRefresher(nfeboard,&FEParamController::refreshNFEBoardProperty,this);

	nfeboard=m_Driver->nfeboard();
	int32 index=0;
	for(int j=0;j<m_Driver->nslot();j++){
		if(m_Driver->boardtype(j)==stereo_elec::FEBOARD){
			slotindex.set(index,j);
			//ici mettre copie de struct FEParam
			index++;
		}
	}
	index=0;
	for(int i=0;i<nfeboard();i++){
		for(int k=0;k<stereo_elec::NBFECHANNEL;k++){
			itotchannel.set(index,index);
			islot.set(index,slotindex(i));
			ichannel.set(index,k);
			switch (k){
			case 0:	namechannel.set(index,m_Driver->namechannel0(slotindex(i))); break;
			case 1:	namechannel.set(index,m_Driver->namechannel1(slotindex(i))); break;
			case 2:	namechannel.set(index,m_Driver->namechannel2(slotindex(i))); break;
			case 3:	namechannel.set(index,m_Driver->namechannel3(slotindex(i))); break;
			case 4:	namechannel.set(index,m_Driver->namechannel4(slotindex(i))); break;
			case 5:	namechannel.set(index,m_Driver->namechannel5(slotindex(i))); break;
			case 6:	namechannel.set(index,m_Driver->namechannel6(slotindex(i))); break;
			case 7:	namechannel.set(index,m_Driver->namechannel7(slotindex(i))); break;
			}
			//ici mettre copie de struct FEParam
			isselected.set(index,false);
			index++;
		}
	}


	registerProgression(m_Driver, &FEParamController::updateProgression, this);


}

void FEParamController::refreshNFEBoardProperty(int32 value) throw (CannotSetValue){
	ntot.resize(value);
	ntail.resize(value);
	pretrig.resize(value);
	cfddelay.resize(value);
	cfdfraction.resize(value);
	trigmask.resize(value);
	trigmode.resize(value);
	slotindex.resize(value);
	itotchannel.resize(stereo_elec::NBFECHANNEL*value);
	islot.resize(stereo_elec::NBFECHANNEL*value);
	ichannel.resize(stereo_elec::NBFECHANNEL*value);
	namechannel.resize(stereo_elec::NBFECHANNEL*value);
	gain10.resize(stereo_elec::NBFECHANNEL*value);
	trigthres.resize(stereo_elec::NBFECHANNEL*value);
	cfdthres.resize(stereo_elec::NBFECHANNEL*value);
	isselected.resize(stereo_elec::NBFECHANNEL*value);
}


/*
 * updateProgression
 */
void FEParamController::updateProgression() {
	// Calculate its own progression to manage retry case
	int32 progression = m_Driver->commandProgression();
	if (progression >= 100) {
		progression = 99;
	}
	commandProgression = progression;
}


/*
 * select channels
 */
void FEParamController::select() {


}

void FEParamController::set() {

}

}

