/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef FEPARAMCONTROLLER_H
#define FEPARAMCONTROLLER_H

#include <Controller.h>
#include "drivers/stereo_elec/StereoElecDriver.h"

namespace stereo_daq {


/*!
 * \brief Commands select to be used as a thread.
 */
class Select: private controller::Command {

public:
	Select(ExperimentController * c) :
		controller::Command(c, "select", &Select::select, this) {
	}

	virtual void select() = 0;
	void selectCommand(bool logging = false) {
		command(logging, true);
	}
};

/*!
 * \brief Commands set to be used as a thread.
 */
class Set: private controller::Command {

public:
	Set(ExperimentController * c) :
		controller::Command(c, "set", &Set::set, this) {
	}

	virtual void set() = 0;
	void setCommand(bool logging = false) {
		command(logging, true);
	}
};


/*!
 * \class FEParamController
 */

class FEParamController: public ExperimentController,
		public Select,
		public Set{

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 *
	 * It is the default constructor called by the creator class ExperimentControllerCreator.
	 *
	 * \param[in] name the name of the controller
	 */
	FEParamController(const std::string& name);

	/*!
	 * \brief Copy constructor
	 *
	 * \param[in] controller the controller to copy
	 */
	FEParamController(const FEParamController& controller);

	/*!
	 * \brief Destructor
	 */
	virtual ~FEParamController();

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	/*!
	 * Properties
	 */
	Property<int32> nfeboard;

	// 1 value for each fe board
	DynamicProperty<int32> ntot;
	DynamicProperty<int32> ntail;
	DynamicProperty<int32> pretrig;
	DynamicProperty<int32> cfddelay;
	DynamicProperty<int32> cfdfraction;
	DynamicProperty<int32> trigmask;
	DynamicProperty<string> trigmode;
	DynamicProperty<int32> slotindex;

	// 8 values for each fe board
	DynamicProperty<int32> itotchannel;
	DynamicProperty<int32> islot;
	DynamicProperty<int32> ichannel;
	DynamicProperty<std::string> namechannel;
	DynamicProperty<bool> gain10;
	DynamicProperty<int32> trigthres;
	DynamicProperty<int32> cfdthres;
	DynamicProperty<bool> isselected;

	Property<string> selectchoice;
	Property<int32> selectslot;
	Property<string> setparam;
	Property<string> setvalue;

protected:


	DriverPtr<stereo_elec::StereoElecDriver> m_Driver; //! The driver linked to the controller

	/*!
	* \brief select command
	*/
	virtual void select();

	/*!
	* \brief select command
	*/
	virtual void set();

private:

	/*
	 * local values
	 */
	bool m_StartActivated; //! Started activated flag

	void refreshNFEBoardProperty(int32 value) throw (CannotSetValue);


	/*!
	 * \brief update the progression property from driver one
	 */
	void updateProgression();


};
}
#endif // FEPARAMCONTROLLER_H
