/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENSYX527CONTROLLER_H
#define CAENSYX527CONTROLLER_H

#include <Controller.h>
#include "CAENSYx527Driver.h"

namespace caen_syx527 {

/*!
 * \brief Commands on off to be used as a thread.
 */
class On: private controller::Command {

public:
	On(ExperimentController * c) :
		controller::Command(c, "on", &On::on, this) {
	}

	virtual void on() = 0;
	void onCommand(bool logging = false) {
		command(logging, true);
	}
};

/*!
 * \brief Commands on off to be used as a thread.
 */
class Off: private controller::Command {

public:
	Off(ExperimentController * c) :
		controller::Command(c, "off", &Off::off, this) {
	}

	virtual void off() = 0;
	void offCommand(bool logging = false) {
		command(logging, true);
	}
};

/*!
 * \brief Commands select to be used as a thread.
 */
class Select: private controller::Command {

public:
	Select(ExperimentController * c) :
		controller::Command(c, "select", &Select::select, this) {
	}

	virtual void select() = 0;
	void selectCommand(bool logging = false) {
		command(logging, true);
	}
};

/*!
 * \brief Commands set to be used as a thread.
 */
class Set: private controller::Command {

public:
	Set(ExperimentController * c) :
		controller::Command(c, "set", &Set::set, this) {
	}

	virtual void set() = 0;
	void setCommand(bool logging = false) {
		command(logging, true);
	}
};


/*!
 * \class CAENSYx527Controller
 */

class CAENSYx527Controller: public ExperimentController,
		public controller::Stoppable,
		public controller::Read,
		public On,
		public Off,
		public Select,
		public Set{

public:

	//! Type of controller
	static const std::string TYPE;

	/*!
	 * \brief Constructor
	 *
	 * It is the default constructor called by the creator class ExperimentControllerCreator.
	 *
	 * \param[in] name the name of the controller
	 */
	CAENSYx527Controller(const std::string& name);

	/*!
	 * \brief Copy constructor
	 *
	 * \param[in] controller the controller to copy
	 */
	CAENSYx527Controller(const CAENSYx527Controller& controller);

	/*!
	 * \brief Destructor
	 */
	virtual ~CAENSYx527Controller();

	/*!
	 * \brief Method called before changing the property value
	 *
	 * This method is called after setting configuration during the creation of controller.
	 */
	virtual void postConfiguration();

	/*!
	 * Properties
	 */

	Property<int32> ntotchannel;
	DynamicProperty<int32> itotchannel;  //channel index for all slots
	DynamicProperty<int32> islot;		//slot index
	DynamicProperty<int32> ichannel;	//channel index for each slot
	DynamicProperty<std::string> namechannel;
	DynamicProperty<float64> i0set;
	DynamicProperty<float64> v0set;
	DynamicProperty<float64> imon;
	DynamicProperty<float64> vmon;
	DynamicProperty<std::string> power;
	DynamicProperty<std::string> statuschannel;
	DynamicProperty<float64> rampup;
	DynamicProperty<float64> rampdown;
	DynamicProperty<bool> isselected;

	Property<string> selectchoice;
	Property<int32> selectslot;
	Property<string> setparam;
	Property<string> setvalue;
	Property<float64> vmax;

protected:

	/*!
	 * \brief format method
	 * Format current value using number of decimal property
	 * \param[in] value the current value
	 * \return the string of the position
	 */
	std::string currentFormatMethod(float64 value);

	/*!
	 * \brief format method
	 * Format voltage value using number of decimal property
	 * \param[in] value the voltage value
	 * \return the string of the position
	 */
	std::string voltageFormatMethod(float64 value);

	DriverPtr<CAENSYx527Driver> m_Driver; //! The driver linked to the controller

	/*!
	 * \brief Read command
	 */
	virtual void read();

	/*!
	 * \brief Start command
	 */
	virtual void start();

	/*!
	 * \brief Stop command
	 */
	virtual void stop();

	/*!
	 * \brief On command
	 */
	virtual void on();

	/*!
	 * \brief Off command
	 */
	virtual void off();

	/*!
	* \brief select command
	*/
	virtual void select();

	/*!
	* \brief select command
	*/
	virtual void set();

private:

	/*
	 * local values
	 */
	bool m_StartActivated; //! Started activated flag



	void refreshNTotalChannelProperty(int32 value) throw (CannotSetValue);
	void refreshChannelNameProperty(int32 index, const std::string& value) throw (CannotSetValue);
	void refreshV0SetProperty(int32 index,float64 value) throw (CannotSetValue);
	void refreshI0SetProperty(int32 index,float64 value) throw (CannotSetValue);
	void refreshVMaxProperty(float64 value) throw (CannotSetValue);
	void refreshRampUpProperty(int32 index,float64 value) throw (CannotSetValue);
	void refreshRampDownProperty(int32 index,float64 value) throw (CannotSetValue);
	void refreshPowerProperty(int32 index,const std::string& value) throw (CannotSetValue);

	/*!
	 * \brief update the actual current property from driver one
	 */
	virtual void updateIMon(int32 index);

	/*!
	 * \brief update the actual current property from driver one
	 */
	virtual void updateVMon(int32 index);

	/*!
	 * \brief update the channel nam property from driver one
	 */
	void updateChannelName(int32 index);

	/*!
	 * \brief update the set voltage property from driver one
	*/
	void updateV0Set(int32 index);

	/*!
	* \brief update the set voltage property from driver one
	*/
	void updateI0Set(int32 index);

	/*!
	* \brief update the set voltage property from driver one
	*/
	void updateRampUp(int32 index);
	/*!
	* \brief update the set voltage property from driver one
	*/
	void updateRampDown(int32 index);

	/*!
	* \brief update the set voltage property from driver one
	*/
	void updatePower(int32 index);

	/*!
	 * \brief update the status property from driver one
	 */
	void updateChannelStatus(int32 index);

	/*!
	 * \brief update the progression property from driver one
	 */
	void updateProgression();


	//! Current unit
	static const std::string CURRENT_UNIT;
	//! Voltage unit
	static const std::string VOLTAGE_UNIT;
};
}
#endif // CAENSYX527CONTROLLER_H
